false = False
true = True
null = None
mime_type_table = [
  {
    "name": "application/vnd.lotus-1-2-3",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".123"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Lotus_1-2-3"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.lotus-1-2-3"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.in3d.3dml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".3dml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/3gpp2",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [
        "audio/3gpp2"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".3g2"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/3G2"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/video/3gpp2"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/avif",
    "description": "AVIF (AV1 Image File Format) is an image and animation file format, an extension of <a href=\"/image/heif\">HEIF</a> allowing images encoded with AV1. Most files will only have AV1-encoded images, but a mixture of different encodings is allowed. Early draft versions were named AV1 Still Image File Format.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/AVIF"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/image/heif"
      },
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/AVIF"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [
        "image/avif-sequence"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".avif"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },{
    "name": "image/avif-sequence",
    "description": "AVIF (AV1 Image File Format) is an image and animation file format, an extension of <a href=\"/image/heif\">HEIF</a> allowing images encoded with AV1. Most files will only have AV1-encoded images, but a mixture of different encodings is allowed. Early draft versions were named AV1 Still Image File Format.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/AVIF"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/image/heif"
      },
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/AVIF"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [
        "image/avif"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".avifs"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-krita",
    "description": "KRA is the file format for Krita, a raster graphics editor. It is a ZIP archive containing a number of files, including the image data, the layer structure, and the document settings.<br /><br />It is similar in function to PSD files for photoshop.<br /><br />A .krz file is a compressed version of a .kra file and only missing the mergedimage.png contained within it to save storage. The lack of this file can affect interchange with other applications such as Scribus.",
    "furtherReading": [
      {
        "title": "Krita File Format",
        "url": "https://docs.krita.org/en/general_concepts/file_formats/file_kra.html"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Krita"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kra",
      ".krz"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/heic",
    "description": "The MIME subtype name may be 'heic' only if the file conforms to the requirements of the 'heic', 'heix', 'heim', or 'heis' brand, and contains at least one of those brands as a compatible brand. The MIME subtype name may be 'heif' only if the file conforms to the requirements of the 'mif1' brand, and contains that brand as a compatible brand.<br /><br />HEIF (High Efficiency Image File Format) is the lossy image and animation format defined by MPEG-H Part 12 (ISO/IEC 23008-12). <br /><br />It is closely related to the HEVC video format, so in that way it is similar to BPG. It uses boxes/atoms format as a container format, so in that way it is similar to JPEG 2000.",
    "furtherReading": [
      {
        "title": "The Moving Picture Experts Group",
        "url": "https://mpeg.chiariglione.org/standards/mpeg-h/image-file-format"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/HEIF"
      },
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/High_Efficiency_Image_File_Format"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/image/heic"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [
        "image/heif"
      ],
      "parentOf": [],
      "alternativeTo": [
        "image/heif"
      ]
    },
    "fileTypes": [
      ".heif",
      ".heic"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/3gpp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [
        "audio/3gpp"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".3gp"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/3GPP"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/video/3gpp"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/3gpp2",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [
        "video/3gpp2"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".3g2"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/3G2"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/audio/3gpp2"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-7z-compressed",
    "description": "7z is a file format for archiving and data compression, developed by Igor Pavlov. It is the native format of the 7-Zip archiving application.",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".7z"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/7z"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/octet-stream",
    "description": "This MIME type is used for binary files, it's usually a fallback for unknown/generic MIMEs with no specific designation.",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".a",
      ".bin",
      ".bpk",
      ".deploy",
      ".dist",
      ".distz",
      ".dmg",
      ".dms",
      ".dump",
      ".elc",
      ".lha",
      ".lrf",
      ".lzh",
      ".o",
      ".obj",
      ".pkg",
      ".so"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-authorware-bin",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aab",
      ".u32",
      ".vox",
      ".x32"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-icns",
    "description": "ICNS (often written in lowercase as icns) is the native icon format of Mac OS and OS X.<br /><br />It goes by many other names, including Apple Icon, Macintosh Icon, Mac OS Icon, Macintosh OS X Icon, Mac OS X Icon Resource, Mac OS icns Resource, and IconFamily Resource.<br /><br />There are usually multiple images in a file. Supported image sizes are 16×12, and 16, 32, 48, 128, 256, 512, and 1024 pixels square.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/ICNS"
      },
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/Apple_Icon_Image_format"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".icns"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/mp4",
    "description": "MP4 is a multimedia container format defined by MPEG-4 Part 14 (ISO/IEC 14496-14). It is based on the Apple QuickTime format, but is not identical to it.<br /><br />It is a successor to the older MOV format, and is used for video (with H.264 and H.265 codecs) and audio (with AAC and AC-3 codecs).<br /><br />It is used by iTunes for music and video downloads, and by many other services for video streaming. It is also used for video files on Blu-ray discs.",
    "furtherReading": [
      {
        "title": "RFC Editor - RFC4337",
        "url": "https://www.rfc-editor.org/rfc/rfc4337"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/MP4"
      },
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/MPEG-4_Part_14"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/audio/mp4"
      }
    ],
    "links": {
      "deprecates": [
        "audio/mpeg4-generic",
        "audio/x-m4a",
        "audio/m4a"
      ],
      "relatedTo": [
        "video/mp4",
        "audio/aac",
        "audio/aacp",
        "audio/3gpp",
        "audio/3gpp2",
        "audio/mp4a-latm",
        "audio/mpeg4-generic"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mp4",
      ".m4a",
      ".m4b",
      ".m4p",
      ".m4r",
      ".m4v",
      ".mp4v",
      ".3gp",
      ".3g2",
      ".3ga",
      ".3gpa",
      ".3gpp",
      ".3gpp2",
      ".3gp2"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/aac",
    "description": "AAC (Advanced Audio Coding) is a compressed audio format defined in MPEG-2 Part 7 (ISO/IEC 13818-7), and in an updated form in MPEG-4 Part 3 (ISO/IEC 14496-3). It was designed to be the successor to MP3. iTunes distributes (or distributed) files in this format, apparently including various metadata (like the name, and sometimes email of the account which downloaded it, along with the time it was downloaded).<br /><br />An AAC file may contain only the raw AAC format, or it may use a multimedia container format such MP4 or QuickTime. AAC is often used for the audio component of a video file.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/AAC"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/audio/aac"
      }
    ],
    "links": {
      "deprecates": [
        "audio/x-aac"
      ],
      "relatedTo": [
        "audio/aacp"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aac",
      ".m4a"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/mp4a-latm",
    "description": "Registered by RFC 3016; updated by RFC 6416, published October 2011)",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/audio/mp4a-latm"
      },
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/MPEG-4_Part_3#MPEG-4_Audio_Lossless_Coding_(ALS)"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [
        "audio/aac",
        "audio/aacp",
        "audio/3gpp",
        "audio/3gpp2",
        "audio/mp4",
        "audio/mpeg4-generic"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [],
    "notices": {
      "hasNoOfficial": true,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/aacp",
    "description": "AAC+ (Advanced Audio Coding Plus) is a compressed audio format defined in MPEG-4 Part 3 (ISO/IEC 14496-3). It is a successor to AAC, and is designed to be more efficient than AAC for low bitrates. It is also known as HE-AAC (High Efficiency AAC).<br /><br />An AAC+ file may contain only the raw AAC+ format, or it may use a multimedia container format such MP4 or QuickTime. AAC+ is often used for the audio component of a video file.<br /><br />Differs from audio/aac and has lower browser compatibility for playback. The Wikipedia link has the most information I could find.",
    "furtherReading": [
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/High-Efficiency_Advanced_Audio_Coding"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/AAC"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [
        "audio/aac"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aacp"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": true,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-authorware-map",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aam"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-authorware-seg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aas"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/AAS"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-abiword",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".abw",
      ".zabw",
      ".abw.gz"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/ABW"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.americandynamics.acc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".acc"
    ],
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.americandynamics.acc"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-ace-compressed",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ace"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/ACE"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.acucobol",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".acu"
    ],
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.acucobol"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.acucorp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".acutc",
      ".atc"
    ],
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.acucorp"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/adpcm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".adp"
    ],
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/ADPCM"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.audiograph",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aep"
    ],
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.audiograph"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-font-type1",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".afm",
      ".pfa",
      ".pfb",
      ".pfm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ibm.modcap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".afp",
      ".list3820",
      ".listafp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/postscript",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ai",
      ".eps",
      ".ps"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.adobe.air-application-installer-package+zip",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".air"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.amiga.ami",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ami"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.android.package-archive",
    "description": "APK is an archive format used for distributing Android apps. It is based on the Jar format (for Java), and like that format, is actually a ZIP archive with a different extension, and with specific files and directories within it.<br /><br />The executable part of the app is usually in either Dalvik Executable (for older Android versions prior to 5.0) or ART format (for newer versions).<br /><br />Metadata about the app is in a META-INF directory within the archive (similarly to Jar files), compiled code is in a lib directory, and resources in a res directory. There are also a few files at the root level including AndroidManifest.xml (which may be regular XML or, often, binary XML).",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/APK"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".apk"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-ms-application",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".application"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.lotus-approach",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".apr"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pgp-signature",
    "description": "PGP (Pretty Good Privacy) is an encryption program, and its encryption format which eventually became the OpenPGP standard.<br /><br />Created by Phil Zimmerman in 1991, it got into some legal trouble early on because encryption of this grade was classified as a munition under U.S. law and restricted from export. These laws, while not completely repealed even now, have been considerably liberalized since, and now export of PGP and related software is only restricted when it's being exported to countries such as Iran that are under special sanction. Zimmerman's PGP company was eventually acquired by Network Associates (now McAfee), which eventually rebranded the original command-line-based PGP as \"McAfee E-Business Server\" and sold off all other PGP assets to some PGP developers who formed a new PGP company, which was eventually acquired by Symantec. Meanwhile, the specifications of the file format had been released as the OpenPGP spec and published as an RFC document, making it available for use in other products.<br /><br />GNU Privacy Guard (GnuPG, GPG) is a commonly-used encryption tool using PGP format, running on Linux systems.<br /><br />PGP uses a combination of several encryption techniques applied serially, including symmetric-key and public-key cryptography.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/PGP"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".asc",
      ".sig"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-ms-asf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".asf",
      ".asx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-asm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".asm",
      ".s"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.accpac.simply.aso",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aso"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/atom+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".atom"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/atomcat+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".atomcat"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/atomsvc+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".atomsvc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.antix.game-component",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".atx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/basic",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".au",
      ".snd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-msvideo",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".avi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/applixware",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.airzip.filesecure.azf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".azf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.airzip.filesecure.azs",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".azs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.amazon.ebook",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".azw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msdownload",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".bat",
      ".com",
      ".dll",
      ".exe",
      ".msi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-bcpio",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".bcpio"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-font-bdf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".bdf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.syncml.dm+wbxml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".bdm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fujitsu.oasysprs",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".bh2"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.bmi",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".bmi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/bmp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".bmp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.framemaker",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".book",
      ".fm",
      ".frame",
      ".maker"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.previewsystems.box",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".box"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-bzip2",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".boz",
      ".bz2"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/prs.btif",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".btif"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-bzip",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".bz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-c",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".c",
      ".cc",
      ".cpp",
      ".cxx",
      ".dic",
      ".h",
      ".hh"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.clonk.c4group",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".c4d",
      ".c4f",
      ".c4g",
      ".c4p",
      ".c4u"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-cab-compressed",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cab"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.curl.car",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".car"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-pki.seccat",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cat"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-director",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cct",
      ".cst",
      ".cxt",
      ".dcr",
      ".dir",
      ".dxr",
      ".fgd",
      ".swa",
      ".w3d"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/ccxml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ccxml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.contact.cmsg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cdbcmsg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-netcdf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cdf",
      ".nc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mediastation.cdkey",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cdkey"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "chemical/x-cdx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cdx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.chemdraw+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cdxml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.cinderella",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cdy"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pkix-cert",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cer"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/cgm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cgm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-chat",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".chat"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-htmlhelp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".chm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kde.kchart",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".chrt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "chemical/x-cif",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cif"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.anser-web-certificate-issue-initiation",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cii"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-artgalry",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cil"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.claymore",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cla"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/java-vm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".class"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.crick.clicker.keyboard",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".clkk"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.crick.clicker.palette",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".clkp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.crick.clicker.template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".clkt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.crick.clicker.wordbank",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".clkw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.crick.clicker",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".clkx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msclip",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".clp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.cosmocaller",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cmc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "chemical/x-cmdf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cmdf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "chemical/x-cml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.yellowriver-custom-menu",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cmp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-cmx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cmx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.rim.cod",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cod"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/plain",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [
        "text/x-log"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".conf",
      ".def",
      ".diff",
      ".in",
      ".ksh",
      ".list",
      ".log",
      ".pl",
      ".text",
      ".txt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.debian.binary-package",
    "description": "deb (Debian package) is a file format used for software distribution in the Debian Linux platform. It is actually an AR archive, with content that includes some tar files. <br /><br />The \"udeb\" files are in the same basic format as \"deb\", but are considered \"micro deb\", used for smaller packages with only the most essential files.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Deb"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".deb",
      ".udeb"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/markdown",
    "description": "Markdown is a lightweight and human readable markup format for text formatting created by John Gruber and Aaron Swartz. It is similar to various forms of wiki markup. There is no formal specification for the original Markdown, and it has ambiguities that are handled inconsistently by different implementations.<br /><br />An attempt to improve on this situation was done (released 2014-09) by a group unrelated to the originators of Markdown, and was originally dubbed Standard Markdown until John Gruber objected to this name, and it was first renamed \"Common Markdown\" and later CommonMark. Markdeep is an extended Markdown implemented in JavaScript for web use.",
    "furtherReading": [
      {
        "title": "RFC 7763 - The text/markdown Media Type",
        "url": "https://tools.ietf.org/html/rfc7763"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Markdown"
      }
    ],
    "links": {
      "deprecates": [
        "text/x-markdown"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".md",
      ".markdown",
      ".mdown",
      ".markdn"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/wasm",
    "description": "A WASM binary allowed to be used with WebAssembly.instantiateStreaming.",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/wasm"
      },
      {
        "title": "W3C - Streaming Module Compilation and Instantiation",
        "url": "https://www.w3.org/TR/wasm-web-api-2/#streaming-modules"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wasm"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-cpio",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cpio"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/mac-compactpro",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cpt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-mscardfile",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".crd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pkix-crl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".crl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-x509-ca-cert",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".crt",
      ".der"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-csh",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".csh"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "chemical/x-csml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".csml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.commonspace",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".csp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/css",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".css"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/csv",
    "description": "CSV (Comma Separated Values) is a text-based format typically used for the storage or exchange of database-like records. In Microsoft Windows systems, it will commonly open in Excel, but it is not a proprietary format and can be used in many other programs.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/CSV"
      }
    ],
    "links": {
      "deprecates": [
        "application/csv",
        "text/x-csv",
        "application/x-csv",
        "text/x-comma-separated-values",
        "text/comma-separated-values"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".csv"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/cu-seeme",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cu"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.curl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".curl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/prs.cww",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cww"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mobius.daf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".daf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fdsn.seed",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dataless",
      ".seed"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/davmount+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".davmount"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.curl.dcurl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dcurl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oma.dd2+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dd2"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fujixerox.ddd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ddd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-debian-package",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".deb",
      ".udeb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.dreamfactory",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dfac"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mobius.dis",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dis"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.djvu",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".djv",
      ".djvu"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.dna",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dna"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/msword",
    "description": "Microsoft Word is a popular word processor program which has existed in a number of versions for DOS, Windows, and the Macintosh. It is often distributed as part of the Microsoft Office suite. Various native file formats have been used, and some other non-Word-specific formats can also be opened and saved.<br /><br />Please note the more modern .docx file extension does not use this mime type.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Microsoft_Word"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".doc",
      ".dot",
      ".wiz"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-word.document.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".docm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".docx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-word.template.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dotm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openxmlformats-officedocument.wordprocessingml.template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dotx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.osgi.dp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.dpgraph",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dpg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/prs.lines.tag",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dsc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-dtbook+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dtb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/xml-dtd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dtd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/vnd.dts",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dts"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/vnd.dts.hd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dtshd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-dvi",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dvi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "model/vnd.dwf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dwf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.dwg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dwg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.dxf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dxf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.spotfire.dxp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dxp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/vnd.nuera.ecelp4800",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ecelp4800"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/vnd.nuera.ecelp7470",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ecelp7470"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/vnd.nuera.ecelp9600",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ecelp9600"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/ecmascript",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ecma"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.novadigm.edm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".edm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.novadigm.edx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".edx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.picsel",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".efif"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.pg.osasli",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ei6"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "message/rfc822",
    "description": "A Content-Type of \"message/rfc822\" indicates that the body contains an encapsulated message, with the syntax of an RFC 822 message.",
    "furtherReading": [
      {
        "title": "W3C",
        "url": "https://www.w3.org/Protocols/rfc1341/7_3_Message.html"
      },
      {
        "title": "Internet Engineering Task Force",
        "url": "https://www.ietf.org/rfc/rfc0822.txt"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".eml",
      ".mht",
      ".mhtml",
      ".mime",
      ".nws"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/emma+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".emma"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/vnd.digital-winds",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".eol"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-fontobject",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".eot"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/epub+zip",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".epub"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.eszigno3+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".es3",
      ".et3"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.epson.esf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".esf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-setext",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".etx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.novadigm.ext",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ext"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/andrew-inset",
    "description": "Published specification:  \"Multimedia Applications Development with the Andrew Toolkit\", by Nathaniel S. Borenstein, Prentice Hall, 1990. <br /><br />Editors note: I looked him up, he works at Mimecast now. Reach out!",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/andrew-inset"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ez"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ezpix-album",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ez2"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ezpix-package",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ez3"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-fortran",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".f",
      ".f77",
      ".f90",
      ".for"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-f4v",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".f4v"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.fastbidsheet",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fbs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fdf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fdf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.denovo.fcselayout-link",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fe_launch"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fujitsu.oasysgp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fg5"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-freehand",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fh",
      ".fh4",
      ".fh5",
      ".fh7",
      ".fhc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-xfig",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fig"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-fli",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fli"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.micrografx.flo",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".flo"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-flv",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".flv"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kde.kivio",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".flw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.fmi.flexstor",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".flx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.fly",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fly"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.frogans.fnc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fnc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.fpx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fpx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fsc.weblaunch",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fsc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.fst",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fst"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fluxtime.clip",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ftc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.anser-web-funds-transfer-initiation",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fti"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/vnd.fvt",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fvt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fuzzysheet",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".fzs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/g3fax",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".g3"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.groove-account",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gac"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "model/vnd.gdl",
    "description": "",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/model/vnd.gdl"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gdl"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.dynageo",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".geo"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.geometry-explorer",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gex",
      ".gre"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.geogebra.file",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ggb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.geogebra.tool",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ggt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.groove-help",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ghf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/gif",
    "description": "Graphics Interchange Format (GIF) was introduced by the CompuServe online service in 1987, intended to provide a consistent and compact format for graphics to be downloaded on that service. Since the specifications were openly released, the format gained wide use in graphics software and on online services and bulletin board systems (BBSs), not just CompuServe; later it became a major Web graphic format. GIF's ability to have animation (unlike most still graphic formats) has caused it to gain some Internet notoriety and use in conjunction with \"memes\".",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gif"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.groove-identity-message",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gim"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.gmx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gmx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-gnumeric",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gnumeric"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.flographit",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gph"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.grafeq",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gqf",
      ".gqs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/srgs",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gram"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.groove-injector",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".grv"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/srgs+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".grxml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-font-ghostscript",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gsf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-gtar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gtar"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.groove-tool-message",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gtm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "model/vnd.gtw",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gtw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.graphviz",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gv"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-gzip",
    "description": "",
    "links": {
      "deprecates": [
        "application/gzip"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gz",
      ".tgz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/gzip",
    "description": "",
    "links": {
      "deprecates": [
        "application/x-gzip"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gz",
      ".tgz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/h261",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".h261"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "gcode",
    "description": "GCODE files are created by slicing programs, such as Simplify3D and Slic3r, that translate CAD drawings into G-Code, which a 3D printer can read.<br /><br />There's no official mimetype assigned as far as the author is aware, however these are the ones commonly reported.",
    "links": {
      "deprecates": [
        "text/x.gcode",
        "text/x-gcode"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gcode"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/h263",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".h263"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/h264",
    "description": "Advanced Video Coding (AVC), also referred to as H.264 or MPEG-4 Part 10, Advanced Video Coding (MPEG-4 AVC), is a video compression standard based on block-oriented, motion-compensated integer-DCT coding.[1] It is by far the most commonly used format for the recording, compression, and distribution of video content, used by 91% of video industry developers as of September 2019.<br /><br />It supports resolutions up to and including 8K UHD.<br /><br />H.264 is controversial for being patent-encumbered, and hence subject to royalty requirements. In 2013, Cisco is attempting to partially remedy this by producing a freely distributed executable H.264 codec for many platforms for which they have paid the royalty, allowing anybody to download and use it in unmodified form royalty-free. The source code is also openly available, but any altered versions that anybody might create from it would require separate licensing, as would any distribution of the executables other than direct download from Cisco. Thus, the only way to use it as part of a product without additional license fees is to have the product's installer download the executable from Cisco during the install process, rather than including it directly in your own product (whether on disk or downloadable from your site).<br /><br />HEVC (H.265) has been developed as a more efficient successor to this format, but it is also patent-encumbered. Attempts to create a royalty-free alternative include AV1, Daala, VP9, and Thor.<br /><br />The H.264 specification does not, in an extremely strict sense, define a codec which fits into a series of bytes; instead, it describes the stream as fitting into a sequence of frames called \"network abstraction layer units\", which can then be delineated into a raw byte format by the user. The format for putting these into a raw bytestream described by Annex B of the specification apparently is apparently very widely-used.",
    "furtherReading": [
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/Advanced_Video_Coding"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/H.264"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".h264"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hbci",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hbci"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.gerber",
    "description": "The Gerber file format is the de facto standard for printed circuit board (PCB) image data transfer. Gerber files consist of 7bit ASCII text and do not contain any active or executable content. <br /><br />The format does include an optional integrity check: A checksum of the file's content can be specified using the MD5 File Attribute. Any additional integrity protection or privacy protection must be supplied externally.",
    "furtherReading": [
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/Gerber_format"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.gerber"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".gbr"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-hdf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hdf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/winhlp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hlp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hp-hpgl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hpgl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hp-hpid",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hpid"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hp-hps",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hps"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/mac-binhex40",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hqx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kenameaapp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".htke"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/html",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".htm",
      ".html"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.yamaha.hv-dic",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hvd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.yamaha.hv-voice",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hvp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.yamaha.hv-script",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".hvs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.iccprofile",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".icc",
      ".icm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "x-conference/x-cooltalk",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ice"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-icon",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ico"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/calendar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ics",
      ".ifb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/ief",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ief"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.shana.informed.formdata",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ifm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "model/iges",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".iges",
      ".igs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.igloader",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".igl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.micrografx.igx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".igx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.shana.informed.interchange",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".iif"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.accpac.simply.imp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".imp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-ims",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ims"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.shana.informed.package",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ipk"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ibm.rights-management",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".irm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.irepository.package+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".irp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.shana.informed.formtemplate",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".itp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.immervision-ivp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ivp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.immervision-ivu",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ivu"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.sun.j2me.app-descriptor",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jad"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.jam",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jam"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/java-archive",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jar"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-java-source",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".java"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.jisp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jisp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hp-jlyt",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jlt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-java-jnlp-file",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jnlp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.joost.joda-archive",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".joda"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/jpeg",
    "description": "JPEG is a popular raster image format well-suited to photographic images. It usually uses lossy DCT compression. It is named after the Joint Photographic Experts Group, the organization which developed the format. It is sometimes called JPEG1, JPEG-1, or JPEG 1992 to help disambiguate it.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/JPEG"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jpe",
      ".jpeg",
      ".jpg",
      ".pjpg",
      ".jfif",
      ".jfif-tbnl",
      ".jif"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/pjpeg",
    "description": "PJPEG files refer to images wherein the Progressive JPEG format (hence JPEG) is used for bitmap images.",
    "furtherReading": [
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/JPEG"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jpe",
      ".jpeg",
      ".jpg",
      ".pjpg",
      ".jfi",
      ".jfif",
      ".jfif-tbnl",
      ".jif"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/jpm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jpgm",
      ".jpm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/jpeg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".jpgv"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-trash",
    "description": "Why even drag this in?",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-shellscript",
    "description": "Why even drag this in?",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sh"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/javascript",
    "description": "JavaScript (sometimes abbreviated JS) is a scripting language commonly implemented as part of a web browser in order to create enhanced user interfaces and dynamic websites, but also used in other contexts (such as server-side JavaScript).<br /><br />This is not the same as <a href=\"/text/x-java-source\">Java</a>. People constantly get confused about that.<br /><br />JavaScript was originally developed at Netscape by Brendan Eich (who later became Mozilla CEO... very briefly), where it was originally called LiveScript while under development, but became JavaScript (with the name licensed from Sun) in order to capitalize on the popularity of Java, though the languages aren't really related (although there are some similarities in syntax). Later, a Microsoft implementation designed to be (more or less) compatible was called JScript, and an attempt at a formally standardized version of the language was published by ECMA as ECMAScript.<br /><br />The node.js runtime environment is (mostly) implemented in JavaScript, as are applications running within it.<br /><br />Due to its status as the only scripting language for browsers, JavaScript skyrocketed in popularity in the 2000's and 2010's and as of 2017 serves as a lingua franca of Web development. Arguably, it is currently the world's most popular programming language (it tops GitHub and StackOverflow currently).",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Javascript"
      },
      {
        "title": "WHATWG",
        "url": "https://html.spec.whatwg.org/multipage/scripting.html#scriptingLanguages"
      }
    ],
    "links": {
      "deprecates": [
        "application/ecmascript",
        "application/javascript",
        "application/x-ecmascript",
        "application/x-javascript",
        "text/ecmascript",
        "text/javascript1.0",
        "text/javascript1.1",
        "text/javascript1.2",
        "text/javascript1.3",
        "text/javascript1.4",
        "text/javascript1.5",
        "text/jscript",
        "text/livescript",
        "text/x-ecmascript",
        "text/x-javascript"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".js"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/json",
    "description": "JSON (JavaScript Object Notation) is widely used by web applications, mobile apps, and other programs to communicate between different systems (such as between a client and server). While it is named after JavaScript, there are libraries for many other programming and scripting languages to let them use this format as well. The geospatial format GeoJSON is based on JSON, as is the remote-procedure-call protocol JSON-RPC and the JSON-LD linked-data format. The IBM standard of JSONx is an XML implementation of JSON. I-JSON is a restricted profile of JSON for Internet use. JSON Web Tokens, JSON Web Encryption, and JSON Web Signatures are JSON-based formats for security-related functions.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/JSON"
      },
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/json"
      }
    ],
    "links": {
      "deprecates": [
        "text/json"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".json"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/midi",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kar",
      ".mid",
      ".midi",
      ".rmi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/aiff",
    "description": "AIFF (Audio Interchange File Format) files store uncompressed signed audio samples inside an IFF container. There's also a compressed variant, called AIFC with the same mimetype.<br /><br />AIFF chunks provide native metadata. In addition, although it is not in the specification, an ID3 chunk is often used to include ID3V2 metadata.<br /><br />It is difficult to trace down official mention of an official mimetype declaration, however you should follow existing guidance which is to assign with audio/aiff and support reading from audio/x-aiff also.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/AIFF"
      }
    ],
    "links": {
      "deprecates": [
      ],
      "relatedTo": [],
      "parentOf": [
        "audio/x-aiff"
      ],
      "alternativeTo": []
    },
    "fileTypes": [
      ".aiff",
      ".aif",
      ".aff"
    ],
    "notices": {
      "hasNoOfficial": true,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/opus",
    "description": "Opus audio",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".opus"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kde.karbon",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".karbon"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kde.kformula",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kfo"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kidspiration",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kia"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-killustrator",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kil"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.google-earth.kml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.google-earth.kmz",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kmz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kinar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kne",
      ".knp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kde.kontour",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kon"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kde.kpresenter",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kpr",
      ".kpt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kde.kspread",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ksp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kahootz",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ktr",
      ".ktz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kde.kword",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kwd",
      ".kwt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-latex",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".latex"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.llamagraphics.life-balance.desktop",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".lbd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.llamagraphics.life-balance.exchange+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".lbe"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hhe.lesson-player",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".les"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.route66.link66+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".link66"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/lost+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".lostxml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-lrm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".lrm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.frogans.ltf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ltf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/vnd.lucent.voice",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".lvp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.lotus-wordpro",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".lwp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msmediaview",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".m13",
      ".m14",
      ".mvb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/mpeg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".m1v",
      ".m2v",
      ".mpa",
      ".mpe",
      ".mpeg",
      ".mpg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/mpeg",
    "description": "MP3 is the name commonly given to the audio formats specified by MPEG-1 Layer III and MPEG-2 Layer III, standardized as ISO/IEC 11172-3:1993, with some additions in ISO/IEC 13818-3:1995. It uses lossy compressed data.<br /><br />MP3 is based in part on work by the Fraunhofer Institute, which held patents in the format. Other companies may also have held patents encumbering its implementation. One of the relevant patents was <a href=\"http://www.google.com/patents/US5812672\" target=\"_blank\">U.S. Patent 5,812,672</a>, which expired in September 2015; many other related patents in various countries expired earlier. The final expiration of all relevant patents took until 2017, however, due to some \"submarine patents\" which were kept in the application process for years, extending their expiration dates. A modified version of DCT compression (Discrete cosine transform) is used, a lossy compression method also used in <a href=\"/image/jpeg\">JPEG images</a>.<br /><br />ID3 tags are often used to provide metadata in MP3 files, though they aren't part of the MP3 specification.",
    "furtherReading": [
      {
        "title": "IETF - Internet Engineering Task Force (RFC 3003)",
        "url": "https://tools.ietf.org/html/rfc3003"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/MP3"
      }
    ],
    "links": {
      "deprecates": [
        "audio/mp3",
        "audio/mpeg3",
        "audio/x-mpeg-3"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".m2a",
      ".m3a",
      ".mp2",
      ".mp2a",
      ".mp3",
      ".mpga"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/x-mpegurl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".m3u"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/vnd.mpegurl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".m4u",
      ".mxu"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-m4v",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".m4v"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/mathematica",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ma",
      ".mb",
      ".nb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ecowin.chart",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mag"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/troff",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".man",
      ".me",
      ".ms",
      ".roff",
      ".t",
      ".tr"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/mathml+xml",
    "description": "",
    "links": {
      "deprecates": [
        "text/mathml"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mathml",
      ".mml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/mathml",
    "description": "",
    "links": {
      "deprecates": [
        "application/mathml+xml"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mathml",
      ".mml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sqlite3",
    "description": "SQLite is a self-contained, serverless, zero-configuration, transactional SQL database engine. The code for SQLite is in the public domain and is thus free for use for any purpose, commercial or private.",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.sqlite3"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/SQLite"
      }
    ],
    "links": {
      "deprecates": [
        "application/x-sqlite3"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".db",
      ".sqlite",
      ".sqlite3",
      ".db-wal",
      ".sqlite-wal",
      ".db-shm",
      ".sqlite-shm"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mobius.mbk",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mbk"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/mbox",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mbox"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.medcalcdata",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mc1"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mcd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mcd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.curl.mcurl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mcurl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msaccess",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mdb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.ms-modi",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mdi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "model/mesh",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mesh",
      ".msh",
      ".silo"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mfmp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mfm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.proteus.magazine",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mgz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mif",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mif"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/mj2",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mj2",
      ".mjp2"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.dolby.mlp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mlp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.chipnuts.karaoke-mmd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mmd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.smaf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mmf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.fujixerox.edmics-mmr",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mmr"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msmoney",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mny"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-mobipocket-ebook",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mobi",
      ".prc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/quicktime",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mov",
      ".qt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-sgi-movie",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".movie"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/mp4",
    "description": "MP4 usually refers to the multimedia file format defined in Part 14 (and to a lesser extent Part 1) of the MPEG-4 standard. \"MP4\" could also mean the entire MPEG-4 standard, or some other subset of MPEG-4. MPEG-4 Part 14 is an application of Part 12. The recommended extension for it is \".mp4\", though \".m4a\" is often used for audio-only files. It is standardized by ISO/IEC 14496-14.",
    "links": {
      "deprecates": [],
      "relatedTo": [
        "audio/mp4",
        "audio/aac",
        "audio/aacp",
        "audio/3gpp",
        "audio/3gpp2",
        "audio/mp4a-latm",
        "audio/mpeg4-generic"
      ],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mp4",
      ".mp4v",
      ".mpg4"
    ],
    "furtherReading": [
      {
        "title": "Internet Engineering Task Force (RFC 4337)",
        "url": "https://tools.ietf.org/html/rfc4337"
      },
      {
        "title": "Library of Congress",
        "url": "https://www.loc.gov/preservation/digital/formats/fdd/fdd000426.shtml"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/MP4"
      },
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/MPEG-4_Part_14"
      }
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-iso9660-image",
    "description": "An ISO image is a file containing an image of an optical disc (CD, DVD, etc.), which can be stored and transferred on other media. It is an uncompressed image containing all the data on the CD. (However, audio CDs are unable to be stored in this format because of technical differences in that format; the DDP or CUE and BIN formats can be used instead.) It usually uses the ISO 9660 filesystem, but UDF and Apple Partition Map may also be used.",
    "furtherReading": [
      {
        "title": "Library of Congress",
        "url": "https://www.loc.gov/preservation/digital/formats/fdd/fdd000348.shtml"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/ISO_image"
      },
      {
        "title": "Wikipedia",
        "url": "https://en.wikipedia.org/wiki/ISO_image"
      }
    ],
    "links": {
      "deprecates": [
        "application/x-cd-image"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".iso",
      ".isoimg",
      ".cdr"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/yaml",
    "description": "YAML (YAML Ain’t Markup Language) is a serialization format. It is a strict superset of JSON, with the addition of syntactically significant newlines and indentation, like Python. Unlike Python, however, YAML doesn’t allow literal tab characters for indentation. It is designed to be human-friendly and work well with modern programming languages for common everyday tasks. It is broadly useful for programming needs ranging from configuration files to Internet messaging to object persistence to data auditing. Together with the Unicode standard for characters, it is also well-suited for internationalization.",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/yaml"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/YAML"
      }
    ],
    "links": {
      "deprecates": [
        "text/yaml",
        "text/x-yaml",
        "application/x-yaml"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".yaml",
      ".yml"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/mp4",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mp4s"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mophun.certificate",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mpc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.apple.installer+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mpkg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.blueice.multipass",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mpm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mophun.application",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mpn"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-project",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mpp",
      ".mpt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ibm.minipay",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mpy"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mobius.mqy",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mqy"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/marc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mrc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/mediaservercontrol+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mscml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fdsn.mseed",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mseed"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mseq",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mseq"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.epson.msf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".msf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mobius.msl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".msl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.muvee.style",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".msty"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "model/vnd.mts",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mts"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.musician",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mus"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.recordare.musicxml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".musicxml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mfer",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mwf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/mxf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mxf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.recordare.musicxml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mxl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/xv+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mxml",
      ".xhvml",
      ".xvm",
      ".xvml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.triscape.mxs",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mxs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.nokia.n-gage.symbian.install",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".n-gage"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-dtbncx+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ncx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.nokia.n-gage.data",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ngdat"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.neurolanguage.nlu",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".nlu"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.enliven",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".nml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.noblenet-directory",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".nnd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.noblenet-sealer",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".nns"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.noblenet-web",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".nnw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.net-fpx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".npx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.lotus-notes",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".nsf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fujitsu.oasys2",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oa2"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fujitsu.oasys3",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oa3"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fujitsu.oasys",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oas"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msbinder",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".obd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/oda",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oda"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.database",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".odb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.chart",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".odc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.formula",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".odf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.formula-template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".odft"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.graphics",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".odg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.image",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".odi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.presentation",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".odp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.spreadsheet",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ods"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.text",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".odt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/ogg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oga",
      ".ogg",
      ".spx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-matroska",
    "description": "Matroska (sometimes spelled Matroška) is an open standard multimedia container format. It is based on the EBML metaformat and can be used with a variety of compressed video formats.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/MKV"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mkv"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/x-matroska",
    "description": "Matroska Audio files consist of a Matroska container containing one or more audio files compressed with one of the supported codecs. As well as the compressed audio, a Matroska Audio file might also contain embedded lyrics or transcriptions, in the form of SRT subtitles. In addition, Matroska Audio files can be separated into tracks or chapters - it is possible, for example, to contain a whole album or audiobook in a single file while retaining clear separation between tracks.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Matroska_Audio"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mka"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/ogg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ogv"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/ogg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ogx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/onenote",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".onepkg",
      ".onetmp",
      ".onetoc",
      ".onetoc2"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/oebps-package+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".opf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.palm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oprc",
      ".pdb",
      ".pqa"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.lotus-organizer",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".org"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.yamaha.openscoreformat",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".osf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.yamaha.openscoreformat.osfpvg+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".osfpvg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.chart-template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".otc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "font/woff",
    "description": "Web Open Font Format (WOFF)",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".woff"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "font/woff2",
    "description": "Web Open Font Format (WOFF)",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".woff2"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-redhat-package-manager",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rpa"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-perl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pm",
      ".pl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/webm",
    "description": "WEBM audio",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".weba"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/webm",
    "description": "WebM is a video and audio container format released by Google, used with content encoded under the VP8 or VP9 codecs for video, and Vorbis and Opus for audio. It is a variant of Matroska, based on EBML.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Webm"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".webm"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/webp",
    "description": "WEBP image",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".webp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-font-otf",
    "description": "",
    "links": {
      "deprecates": [
        "font/otf"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".otf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "font/otf",
    "description": "",
    "links": {
      "deprecates": [
        "application/x-font-otf"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".otf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.graphics-template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".otg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.text-web",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oth"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.image-template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oti"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.text-master",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".otm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.presentation-template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".otp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.spreadsheet-template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ots"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.oasis.opendocument.text-template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ott"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openofficeorg.extension",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".oxt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-pascal",
    "description": "Pascal is an influential imperative and procedural programming language, designed in 1968–1969 and published in 1970. Borland's Turbo Pascal, and the later Borland Pascal, were popular in the 1980s and early 1990s on the PC/MS-DOS platform (CP/M versions were also released).",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Pascal"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".p",
      ".pas",
      ".pp",
      ".inc"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pkcs10",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".p10"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-pkcs12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".p12",
      ".pfx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-pkcs7-certificates",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".p7b",
      ".spc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pkcs7-mime",
    "description": "",
    "furtherReading": [
      {
        "title": "IETF - Internet Engineering Task Force (RFC 5273)",
        "url": "https://tools.ietf.org/html/rfc5273.html#page-3"
      },
      {
        "title": "PKI Tutorial",
        "url": "https://pki-tutorial.readthedocs.io/en/latest/mime.html"
      }
    ],
    "links": {
      "deprecates": [
        "application/x-pkcs7-certreqresp",
        "application/x-pkcs7-certificates"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".p7c",
      ".p7m"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-pkcs7-certreqresp",
    "description": "x-pkcs7-certreqresp and the .p7r extension were also introduced by Microsoft. Likely yet another alias for pkcs7-mime.",
    "deprecated": true,
    "useInstead": "application/pkcs7-mime",
    "furtherReading": [
      {
        "title": "IETF - Internet Engineering Task Force (RFC 5273)",
        "url": "https://tools.ietf.org/html/rfc5273.html#page-3"
      },
      {
        "title": "PKI Tutorial",
        "url": "https://pki-tutorial.readthedocs.io/en/latest/mime.html"
      }
    ],
    "links": {
      "deprecates": [
        "application/pkcs7-mime"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".p7r"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pkcs7-signature",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".p7s"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.powerbuilder6",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pbd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-portable-bitmap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pbm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-font-pcf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pcf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hp-pcl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pcl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hp-pclxl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pclxl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-pict",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pct",
      ".pic"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.curl.pcurl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pcurl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-pcx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pcx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pdf",
    "description": "Portable Document Format (PDF) is a document file format originally from Adobe, based on PostScript. It has many subsets.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/PDF"
      }
    ],
    "links": {
      "deprecates": [
        "application/x-pdf"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pdf"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/font-tdpfr",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pfr"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-portable-graymap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pgm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-chess-pgn",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pgn"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pgp-encrypted",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pgp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pkixcmp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pki"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pkix-pkipath",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pkipath"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.3gpp.pic-bw-large",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".plb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mobius.plc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".plc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.pocketlearn",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".plf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pls+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pls"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ctc-posml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/png",
    "description": "Portable Network Graphics (PNG) was devised starting in a discussion on newsgroup comp.graphics in 1995, with the first version of its specification released in 1996. The motivation for its creation was to create a free and unencumbered image format in the wake of the patent issue with GIF.",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/image/png"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/PNG"
      }
    ],
    "links": {
      "deprecates": [
        "image/x-png",
        "image/vnd.mozilla.apng"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".png"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-portable-anymap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pnm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.macports.portpkg",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".portpkg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-powerpoint",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pot",
      ".ppa",
      ".pps",
      ".ppt",
      ".pwz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-powerpoint.template.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".potm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openxmlformats-officedocument.presentationml.template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".potx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-powerpoint.addin.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ppam"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.cups-ppd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ppd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-portable-pixmap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ppm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-powerpoint.slideshow.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ppsm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openxmlformats-officedocument.presentationml.slideshow",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ppsx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-powerpoint.presentation.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pptm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pptx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.lotus-freelance",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pre"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/pics-rules",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".prf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/prql",
    "description": "PRQL is a modern language for transforming data — a simple, powerful, pipelined SQL replacement.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/PRQL"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".prql"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.3gpp.pic-bw-small",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".psb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.adobe.photoshop",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".psd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-font-linux-psf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".psf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.pvi.ptid1",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ptid"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-mspublisher",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pub"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.3gpp.pic-bw-var",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pvb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.3m.post-it-notes",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pwn"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-python",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": ["text/x-python-code", "text/x-python-script", "application/x-python-code", "application/x-python-script"]
    },
    "fileTypes": [
      ".py", ".pyc", ".pyo", ".pyd", ".whl"
    ],
    "furtherReading": [
        {
            "title": "Let's Solve the File Format Problem!",
            "url": "http://fileformats.archiveteam.org/wiki/Python"
        }
    ],
    "notices": {
      "hasNoOfficial": true,
      "communityContributed": false,
      "popularUsage": "text/x-python"
    }
  },
  {
    "name": "audio/vnd.ms-playready.media.pya",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pya"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/vnd.ms-playready.media.pyv",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pyv"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.epson.quickanime",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".qam"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.intu.qbo",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".qbo"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.intu.qfx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".qfx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.publishare-delta-tree",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".qps"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.quark.quarkxpress",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".qwd",
      ".qwt",
      ".qxb",
      ".qxd",
      ".qxl",
      ".qxt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/x-pn-realaudio",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ra",
      ".ram"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.rar",
    "description": "<strong>R</strong>oshal <strong>AR</strong>chive (RAR) is a proprietary file format used by the compression software WinRAR. They make the decompression code available for use in other programs and allow its distribution, but with a license provision that \"You cannot use the unrar source to re-create the RAR compression algorithm, which is proprietary.",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.rar"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/RAR"
      }
    ],
    "links": {
      "deprecates": [
        "application/x-rar-compressed"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rar"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-rar-compressed",
    "description": "<strong>R</strong>oshal <strong>AR</strong>chive (RAR) is a proprietary file format used by the compression software WinRAR. They make the decompression code available for use in other programs and allow its distribution, but with a license provision that \"You cannot use the unrar source to re-create the RAR compression algorithm, which is proprietary.",
    "deprecated": true,
    "useInstead": "application/vnd.rar",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/vnd.rar"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/RAR"
      }
    ],
    "links": {
      "deprecates": [
        "application/vnd.rar"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rar"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-cmu-raster",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ras"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ipunplugged.rcprofile",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rcprofile"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/rdf+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rdf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.data-vision.rdz",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rdz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.businessobjects",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rep"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-dtbresource+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".res"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-rgb",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rgb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/reginfo+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rif"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/resource-lists+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.fujixerox.edmics-rlc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rlc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/resource-lists-diff+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rld"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.rn-realmedia",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/x-pn-realaudio-plugin",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rmp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.jcp.javame.midlet-rms",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rms"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/relax-ng-compact-syntax",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rnc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-rpm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rpm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.nokia.radio-presets",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rpss"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.nokia.radio-preset",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rpst"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/sparql-query",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rq"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/rls-services+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/rsd+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rsd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/rss+xml",
    "description": "The MIME used for RSS feeds (RSS 0.9x, 2.0, 3.0 and others). Many feeds end up being served as plain text/html however and _most_ parsers seem to be able to handle this discrepancy.    RSS is an XML-based document format for the syndication of web\n   content so that it can be republished on other sites or downloaded\n   periodically and presented to users. RSS is currently used for a number of applications, including news headline syndication, weblog content distribution, and the exchange of other timely information such as software release notes.",
    "furtherReading": [
      {
        "title": "RSS Board Media Type Application",
        "url": "https://www.rssboard.org/rss-mime-type-application.txt"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/RSS"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rss",
      ".xml"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/rtf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rtf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/richtext",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".rtx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.yamaha.smaf-audio",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".saf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/sbml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sbml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ibm.secure-container",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msschedule",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".scd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.lotus-screencam",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".scm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/scvp-cv-request",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".scq"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/scvp-cv-response",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".scs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.curl.scurl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".scurl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.stardivision.draw",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sda"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.stardivision.calc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sdc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.stardivision.impress",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sdd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.solent.sdkm+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sdkd",
      ".sdkm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/sdp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sdp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.stardivision.writer",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sdw",
      ".vor"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.seemail",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".see"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sema",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sema"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.semd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".semd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.semf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".semf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/java-serialized-object",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ser"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/set-payment-initiation",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".setpay"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/set-registration-initiation",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".setreg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hydrostatix.sof-data",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sfd-hdstx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.spotfire.sfs",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sfs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.stardivision.writer-global",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sgl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/sgml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sgm",
      ".sgml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-sh",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sh"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-shar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".shar"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/shf+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".shf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.wap.si",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".si"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.wap.sic",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sic"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.symbian.install",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sis",
      ".sisx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-stuffit",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sit"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-stuffitx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sitx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.koan",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".skd",
      ".skm",
      ".skp",
      ".skt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.wap.sl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.wap.slc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".slc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-powerpoint.slide.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sldm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openxmlformats-officedocument.presentationml.slide",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sldx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.epson.salt",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".slt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.stardivision.math",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".smf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/smil+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".smi",
      ".smil"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-font-snf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".snf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.yamaha.smaf-phrase",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".spf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-futuresplash",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".spl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.in3d.spot",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".spot"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/scvp-vp-response",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".spp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/scvp-vp-request",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".spq"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-wais-source",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".src"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/sparql-results+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".srx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.kodak-descriptor",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sse"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.epson.ssf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ssf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/ssml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ssml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.calc.template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".stc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.draw.template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".std"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.wt.stf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".stf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.impress.template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sti"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/hyperstudio",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".stk"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-pki.stl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".stl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.pg.format",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".str"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.writer.template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".stw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sus-calendar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sus",
      ".susp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-sv4cpio",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sv4cpio"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-sv4crc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sv4crc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.svd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".svd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/svg+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".svg",
      ".svgz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-shockwave-flash",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".swf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.arastra.swi",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".swi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.calc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sxc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.draw",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sxd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.writer.global",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sxg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.impress",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sxi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.math",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sxm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.sun.xml.writer",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sxw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.tao.intent-module-archive",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tao"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-tar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tar"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.3gpp2.tcap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tcap"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-tcl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tcl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.smart.teacher",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".teacher"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-tex",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tex"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-texinfo",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".texi",
      ".texinfo"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-tex-tfm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tfm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/tiff",
    "description": "TIFF, formerly known as Tag(ged) Image File Format, is an image format capable of storing multiple high quality images in a single file. <br /><br />A TIFF image may be uncompressed or use a compression scheme internally. Two of the most widely used compression schemes in TIFF files are lossless, including LZW and, for bitonal images CCITT Group 4, as used for facsimile transmission [fax]. JPEG baseline DCT-based lossy compression is also used.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/TIFF"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tif",
      ".tiff"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.tmobile-livetv",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tmo"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-bittorrent",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".torrent"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.groove-tool-template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tpl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.trid.tpt",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tpt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.trueapp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tra"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msterminal",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".trm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/tab-separated-values",
    "description": "",
    "furtherReading": [
      {
        "title": "IANA (Internet Assigned Numbers Authority)",
        "url": "https://www.iana.org/assignments/media-types/text/tab-separated-values"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".tsv"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-font-ttf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ttc",
      ".ttf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.simtech-mindmapper",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".twd",
      ".twds"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.genomatix.tuxedo",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".txd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mobius.txf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".txf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ufdl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ufd",
      ".ufdl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "test/mimetype",
    "description": "This mimetype is a test! Ignore it, it's not real.",
    "links": {
      "deprecates": [
        "test/x-test",
        "test/xx-test"
      ],
      "relatedTo": [
        "mimetype/test",
        "test/mimetype-2"
      ],
      "parentOf": [
        "test/mimetype/test"
      ],
      "alternativeTo": [
        "test/mimetype-2"
      ]
    },
    "fileTypes": [
      ".test"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.umajin",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".umj"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.unity",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".unityweb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.uoml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".uoml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/uri-list",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".uri",
      ".uris",
      ".urls"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-ustar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ustar"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.uiq.theme",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".utz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-uuencode",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".uu"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-cdlink",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vcd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-vcard",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vcf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.groove-vcard",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vcg"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/x-vcalendar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vcs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.vcx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vcx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.visionary",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vis"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/vnd.vivo",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".viv"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "model/vrml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vrml",
      ".wrl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.visio",
    "description": "",
    "furtherReading": [
      {
        "url": "http://msdn.microsoft.com/en-us/library/office/ff768724.aspx",
        "title": "Microsoft's documentation of the Visio file format"
      },
      {
        "url": "https://learn.microsoft.com/en-us/office/client-developer/visio/introduction-to-the-visio-file-formatvsdx",
        "title": "Introduction to the Visio file format (.vsdx)"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vsd",
      ".vss",
      ".vst",
      ".vsw",
      ".vsdx",
      ".vssx",
      ".vstx",
      ".vssm",
      ".vstm"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.vsf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vsf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "model/vnd.vtu",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vtu"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/voicexml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".vxml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-doom",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wad"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/mp2t",
    "description": "MPEG Transport Stream (or MPEG-2 Transport Stream) is an MPEG multimedia format that allows interleaving more than one Packetized Elementary Stream in a single file. It is an alternative to MPEG Program Stream, designed to work better with certain kinds of streaming. It was introduced in the MPEG-2 standard. There is a variant format named M2TS, used with Blu-ray discs.<br /><br />Alternatively, you got here like I did by selecting a typescript file. That's wrong, that's your browser doing that. There's no official standard for typescript.",
    "furtherReading": [
      {
        "url": "http://fileformats.archiveteam.org/wiki/MPEG_Transport_Stream",
        "title": "Let's Solve the File Format Problem!"
      },
      {
        "url": "https://en.wikipedia.org/wiki/MPEG_transport_stream",
        "title": "Wikipedia article on MPEG Transport Stream"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".ts"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/vnd.wav",
    "description": "The Waveform Audio File Format (WAV or WAVE) is a widely used audio format, originally developed by Microsoft and IBM and based on the RIFF wrapper format. The usual audio encoding in a .wav file is LPCM, considered an 'uncompressed' encoding. Because of large file sizes, WAV is not well-suited for distributing audio such as songs or podcasts. WAV is used in MS-Windows to store sounds used in applications. It is also used as an archival format for first-generation (master) files, often with a metadata chunk as specified in the Broadcast Wave (BWF) standard.<br /><br />Note a contributor has discovered compatibility issues with browsers and the official type. See the attached issue for more information.",
    "furtherReading": [
      {
        "url": "http://fileformats.archiveteam.org/wiki/WAV",
        "title": "Let's Solve the File Format Problem!"
      },
      {
        "title": "IETF - Internet Engineering Task Force (RFC 2361)",
        "url": "https://datatracker.ietf.org/doc/html/rfc2361"
      },
      {
        "url": "https://github.com/patrickmccallum/mimetype-io/issues/18",
        "title": "mimetype.io issue #18 discussion on type"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [
        "audio/wav",
        "audio/x-wav",
        "audio/vnd.wave",
        "audio/wave",
        "audio/x-pn-wav"
      ],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wav"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": "audio/wav"
    }
  },
  {
    "name": "audio/x-ms-wax",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wax"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.wap.wbmp",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wbmp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.criticaltools.wbs+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wbs"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.wap.wbxml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wbxml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-works",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wcm",
      ".wdb",
      ".wks",
      ".wps"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-ms-wm",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/x-ms-wma",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wma"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-ms-wmd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wmd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-msmetafile",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wmf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.wap.wml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.wap.wmlc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wmlc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "text/vnd.wap.wmlscript",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wmls"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.wap.wmlscriptc",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wmlsc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-ms-wmv",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wmv"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-ms-wmx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wmx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-ms-wmz",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wmz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.wordperfect",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wpd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-wpl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wpl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.wqd",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wqd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-mswrite",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wri"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/wsdl+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wsdl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/wspolicy+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wspolicy"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.webturbo",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wtb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "video/x-ms-wvx",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".wvx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.hzn-3d-crossword",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".x3d"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-silverlight-app",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xap"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.xara",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xar"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-ms-xbap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xbap"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fujixerox.docuworks.binder",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xbd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-xbitmap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xbm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.syncml.dm+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xdm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.adobe.xdp+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xdp"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.fujixerox.docuworks",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xdw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/xenc+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xenc"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/patch-ops-error+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xer"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.adobe.xfdf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xfdf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.xfdl",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xfdl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/xhtml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xht",
      ".xhtml"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/vnd.xiff",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xif"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-excel",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xla",
      ".xlb",
      ".xlc",
      ".xlm",
      ".xls",
      ".xlt",
      ".xlw"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-excel.addin.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xlam"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-excel.sheet.binary.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xlsb"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-excel.sheet.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xlsm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xlsx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-excel.template.macroenabled.12",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xltm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.openxmlformats-officedocument.spreadsheetml.template",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xltx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xml",
      ".xpdl",
      ".xsl"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.olpc-sugar",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xo"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/xop+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xop"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-xpinstall",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xpi"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-xpixmap",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xpm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.is-xpr",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xpr"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.ms-xpsdocument",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xps"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.intercon.formnet",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xpw",
      ".xpx"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/xslt+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xslt"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.syncml+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xsm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/xspf+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xspf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.mozilla.xul+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xul"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-xwindowdump",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xwd"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "chemical/x-xyz",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".xyz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.zzazz.deck+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".zaz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/zip",
    "description": "ZIP is one of the most popular file compression formats. It was created in 1989 as the native format of the PKZIP program, which was introduced by Phil Katz in the wake of a lawsuit (which he lost) against him by the makers of the then-popular ARC program (and file format) for copyright and trademark infringement in an earlier program PKARC which had been file-compatible with ARC. <br /><br />This resulted in Katz creating a new file format, which rapidly overtook ARC in popularity (to a large extent because of BBS sysops, then the primary users of such compression, resenting the lawsuit). Many programs have been released for a variety of operating systems to compress and decompress ZIP files, and native support for the format is built into several popular operating systems.",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/zip"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/ZIP"
      }
    ],
    "links": {
      "deprecates": [
        "application/x-zip-compressed",
        "application/zip-compressed"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".zip"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/x-zip-compressed",
    "description": "ZIP is one of the most popular file compression formats. It was created in 1989 as the native format of the PKZIP program, which was introduced by Phil Katz in the wake of a lawsuit (which he lost) against him by the makers of the then-popular ARC program (and file format) for copyright and trademark infringement in an earlier program PKARC which had been file-compatible with ARC. <br /><br />This resulted in Katz creating a new file format, which rapidly overtook ARC in popularity (to a large extent because of BBS sysops, then the primary users of such compression, resenting the lawsuit). Many programs have been released for a variety of operating systems to compress and decompress ZIP files, and native support for the format is built into several popular operating systems.",
    "deprecated": true,
    "useInstead": "application/zip",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/zip"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/ZIP"
      }
    ],
    "links": {
      "deprecates": [
        "application/zip",
        "application/zip-compressed"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".zip"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/zip-compressed",
    "description": "ZIP is one of the most popular file compression formats. It was created in 1989 as the native format of the PKZIP program, which was introduced by Phil Katz in the wake of a lawsuit (which he lost) against him by the makers of the then-popular ARC program (and file format) for copyright and trademark infringement in an earlier program PKARC which had been file-compatible with ARC. <br /><br />This resulted in Katz creating a new file format, which rapidly overtook ARC in popularity (to a large extent because of BBS sysops, then the primary users of such compression, resenting the lawsuit). Many programs have been released for a variety of operating systems to compress and decompress ZIP files, and native support for the format is built into several popular operating systems.",
    "deprecated": true,
    "useInstead": "application/zip",
    "furtherReading": [
      {
        "title": "Internet Assigned Numbers Authority (IANA)",
        "url": "https://www.iana.org/assignments/media-types/application/zip"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/ZIP"
      }
    ],
    "links": {
      "deprecates": [
        "application/zip",
        "application/x-zip-compressed"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".zip"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.zul",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".zir",
      ".zirz"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/vnd.handheld-entertainment+xml",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".zmm"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-adobe-dng",
    "description": "DNG (Digital Negative) is a digital camera raw image format developed by Adobe and released in 2004. Adobe also released a free DNG converter at this time which is continually being updated with compatible RAW file types. DNG is an extension of TIFF 6.0 and is compatible with TIFF/EP.<br /><br />Adobe intends for DNG to become an archival file type, creating an open standard for RAW files.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/DNG"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dng"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-sony-arw",
    "description": "Sony ARW is a raw image format used by some Sony digital cameras. It is based on TIFF.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Sony_ARW"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".arw"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-canon-cr2",
    "description": "Canon RAW 2 (CR2) is a raw image format used by Canon cameras. It replaced CRW, starting with the 20D, 350D/Digital Rebel Xt and 1D. It was replaced by Canon RAW 3.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/CR2"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".cr2"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-canon-crw",
    "description": "The Camera Image File Format, sometimes called the Canon RAW (CRW) format, was used by some early Canon digital cameras. It was replaced by the CR2 format starting with the EOS 20D in 2004. The .crw extension is sometimes used by CHDK, but this is a different file format.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Camera_Image_File_Format"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".crw"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-kodak-dcr",
    "description": "This is another proprietary format, used by old Kodak DCS digital cameras. It is based on TIFF. The full-resolution image is in one of the sub-IFDs of the first IFD. A possible way to identify it is that it uses Compression code 65000 in an IFD or sub-IFD.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Kodak"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".dcr"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-epson-erf",
    "description": "ERF files are generated by the Epson R-D1 and R-D1s digital rangefinder cameras.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Epson_ERF"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".erf"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-kodak-k25",
    "description": "K25 format was used by model DC25. High resolution pictures are 140,352 bytes long and Standard resolution is 77,888.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Kodak"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".k25"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-kodak-kdc",
    "description": "Kodak KDC mime and extension can be present for two formats with little in common. See further reading for more information.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Kodak"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".kdc"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-minolta-mrw",
    "description": "MRW (Minolta RAW) is a raw image format used by Minolta (later Konica-Minolta) cameras. After the Sony purchase, Sony branded SLR use a different format: ARW.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Minolta_MRW"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".mrw"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-nikon-nef",
    "description": "Nikon Electronic Format is Nikon's RAW digital image format for DSLR cameras. It is a close relative to TIFF, and has a standard TIFF header. NEF files are usually big-endian, with the exception of files from the Coolpix 5700, which were little-endian. Data can be either 12- or 14-bit (from 2005's D200 onwards), and depending on the camera may be uncompressed, losslessly compressed, or lossily compressed.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Nikon"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".nef"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "application/neo-executable-file",
    "description": "Neo blockchain smart contract executable file.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "https://neo.org"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".nef"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-olympus-orf",
    "description": "ORF, or Olympus RAW, is a raw image format produced by Olympus digital cameras. The format is TIFF-like, but with a different file signature. The full-size image is in the first IFD. There is also Exif data, with a MakerNote.\n\n",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Olympus_ORF"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".orf"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-pentax-pef",
    "description": "Pentax PEF (or Pentax RAW) is a raw image format associated with Pentax digital cameras. It is based on TIFF.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Pentax_PEF"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".pef",
      ".ptx"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-fuji-raf",
    "description": "RAF, or Fujifilm RAW, is a raw image format used by some Fujifilm digital cameras.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Fujifilm_RAF"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".raf"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-panasonic-raw",
    "description": "Panasonic RAW/RW2, also known as Leica RAW or RWL, is a raw image format used by some Panasonic and Leica digital cameras. It can also be known by a camera-specific name such as Lumix RAW or Panasonic LX3 RAW.<br /><br />Different cameras produce different versions of it, and at some point the .raw file extension was changed to .rw2. The term Panasonic RAW might refer to just the .raw files, but more likely includes both .raw and .rw2.",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/Panasonic_RAW/RW2"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".raw",
      ".rw2",
      ".rwl"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "audio/flac",
    "description": "FLAC stands for Free Lossless Audio Codec, an audio format similar to MP3, but lossless, meaning that audio is compressed in FLAC without any loss in quality. <br /><br />It can encode audio with a PCM bit resolution up to 32 bits per sample and sampling rates up to 640 kHz. FLAC-encoded audio is usually found either in a native container (which has the extension .flac), or in an Ogg container (when it's known as OggFLAC).",
    "furtherReading": [
      {
        "title": "The National Archives",
        "url": "https://www.nationalarchives.gov.uk/PRONOM/fmt/279"
      },
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/FLAC"
      }
    ],
    "links": {
      "deprecates": [
        "audio/x-flac"
      ],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".flac"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-sony-sr2",
    "description": "SR2 is a raw image format produced by some makes of Sony cameras. The format is based upon TIFF.",
    "furtherReading": [
      {
        "title": "The National Archives",
        "url": "https://www.nationalarchives.gov.uk/PRONOM/Format/proFormatSearch.aspx?status=detailReport&id=1936&strPageToDisplay=summary"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".sr2"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-sony-srf",
    "description": "",
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".srf"
    ],
    "furtherReading": [],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  },
  {
    "name": "image/x-sigma-x3f",
    "description": "X3F (or Sigma X3F, or Foveon X3F) is a raw image format used by some Sigma digital cameras that use a \"Foveon sensor\".",
    "furtherReading": [
      {
        "title": "Let's Solve the File Format Problem!",
        "url": "http://fileformats.archiveteam.org/wiki/X3F"
      }
    ],
    "links": {
      "deprecates": [],
      "relatedTo": [],
      "parentOf": [],
      "alternativeTo": []
    },
    "fileTypes": [
      ".x3f"
    ],
    "notices": {
      "hasNoOfficial": false,
      "communityContributed": false,
      "popularUsage": null
    }
  }
]
suffix_to_mime_type_dict: dict[str, str] = {}
for d in mime_type_table:
    mime_type = d['name']
    for f in d['fileTypes']:
        suffix_to_mime_type_dict[f] = mime_type